# pyXPathToy
# 
# Evaulates XPath expressions and displays results
# 2006 by Sebastian Schildt
#
# Uses 4Suite XML libraries (http://4suite.org) and
# wxPython bindings (http://www.wxpython.org)
#
# Comments, patches: sebastian@frozenlight.de
#
# Todo: Docs :)


from Ft.Xml.Domlette import NonvalidatingReader,Attr
from Ft.Xml.Domlette import Print, PrettyPrint
from Ft.Xml import ReaderException
from Ft.Lib import Uri
from Ft.Lib import boolean
import Ft

import cStringIO
import sys
from types import *

import wx



#Worker
class XPathWorker:
    gui=None
    doc=None
	
    def __init__(self, thegui):
		self.gui=thegui
		
    def parseXML(self,gui,file):
        print "Parse ",file
        file_uri = Uri.OsPathToUri(file)
        try:
            self.doc = NonvalidatingReader.parseUri(file_uri)
        except ReaderException, inst:
           self.gui.displayError("Error parsing XML file: %s" % (inst))
           return
           
        self.gui.displayInfo("Parsed "+file)
        self.gui.statusbar.PushStatusText("XML file "+file+" loaded")
		
    def evaluateXPath(self, expr):
        if (self.doc==None):
	        self.gui.displayError("Load an XML File first")
	        return
        
        results={}
        try:
             results=self.doc.xpath(expr)
        except SyntaxError, inst:
             self.gui.displayError("Error in XPATH expression: %s" % (inst))
             return

        self.gui.text.Clear()
        self.gui.text.AppendText("Results from matching XPATH expression "+expr+"\n\n")
        
        if type(results)!=ListType: #just got one attribute
            self.gui.text.AppendText(`results`+"\n")
            self.gui.printLine()
            return
                
        #Python list of Domlette nodes       
        for result in results:
            if isinstance(result,Attr): #an attribute
                self.gui.text.AppendText(result.value+"\n")
                self.gui.printLine()
                continue
            buf = cStringIO.StringIO()
            PrettyPrint(result, stream=buf, encoding='us-ascii')
            s = buf.getvalue()
            buf.close()
            self.gui.text.AppendText(s)
            self.gui.printLine()
	
		
   



#GUI class
ID_ABOUT=1
ID_OPEN =2
ID_EXIT =3
ID_EXPR =4
class PyXPath(wx.Frame):    
        
    """ We simply derive a new class of Frame. """
    def __init__(self,parent,id, title):
        wx.Frame.__init__(self,parent,wx.ID_ANY,title,size=(400,600))
   
        #StatusBar
        self.statusbar=self.CreateStatusBar()
        self.statusbar.PushStatusText("No XML file loaded")
        
        #Add result ouput and XPath query line
        self.sizer=wx.BoxSizer(wx.VERTICAL)
        self.innerSizer=wx.BoxSizer(wx.HORIZONTAL)
        self.text = wx.TextCtrl(self,1,style=wx.TE_MULTILINE|wx.TE_READONLY|wx.TE_RICH)
        self.label = wx.StaticText(self, -1, "XPATH expression: ")
        self.xpathExpr = wx.TextCtrl(self, ID_EXPR, "//@title", style=wx.TE_PROCESS_ENTER)
        self.innerSizer.Add(self.label,0,wx.ALIGN_CENTER_VERTICAL)
        self.innerSizer.Add(self.xpathExpr,1,wx.EXPAND)
        
        self.sizer.Add(self.innerSizer,0,wx.EXPAND)
        self.sizer.Add(self.text,2,wx.EXPAND)
        
        
        # Setting up the menu.
        filemenu= wx.Menu()
        filemenu.Append(ID_ABOUT, "&About"," Information about this program")
        filemenu.Append(ID_OPEN, "&Open XML file"," Read and parse XML file")
        
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.
        
        #Connect events
        wx.EVT_MENU(self, ID_OPEN, self.OnOpen) 
        wx.EVT_MENU(self, ID_ABOUT, self.OnAbout)
        wx.EVT_MENU(self, ID_EXIT, self.OnExit) 
        wx.EVT_TEXT_ENTER(self.xpathExpr, ID_EXPR ,self.onEnter)
        
        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        self.sizer.Fit(self)
        
        self.worker=XPathWorker(self)
        self.SetSize((700,400))
        self.Show(True)
        
    def OnAbout(self,e):
        d= wx.MessageDialog( self, 
                            "2006 by Sebastian Schildt\n" 
                            "See http://reparts.org/semantic/ for information","pyXPathToy 0.1", wx.OK) # Create a message dialog box
        d.ShowModal() # Shows it
        d.Destroy() # finally destroy it when finished.
    
    def OnOpen(self, event):
    	""" Open an XML file"""
        self.dirname = ''
        dlg = wx.FileDialog(self, "Choose an XML file", self.dirname, "", "*.xml", wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.filename=dlg.GetFilename()
            self.dirname=dlg.GetDirectory()
            self.worker.parseXML(self,self.dirname+"/"+self.filename)
            
    	
    def onEnter(self,e):
    	self.worker.evaluateXPath(self.xpathExpr.GetValue())
    	
    def printLine(self):
        oldStyle = self.text.GetDefaultStyle()
        self.text.SetDefaultStyle(wx.TextAttr(wx.BLUE, wx.NullColour, wx.NullFont, wx.TEXT_ALIGNMENT_CENTER))
    	self.text.AppendText("----------------------------------------------\n");	
        self.text.SetDefaultStyle(wx.TextAttr(wx.BLACK, wx.NullColour, wx.NullFont, wx.TEXT_ALIGNMENT_LEFT))

 
    def displayInfo(self,msg):    
    	infoDlg = wx.MessageDialog(self,msg,"Information", wx.OK)
        infoDlg.ShowModal()
        infoDlg.Destroy() 
        
    def displayError(self,msg):    
    	infoDlg = wx.MessageDialog(self,msg,"Error", wx.OK)
        infoDlg.ShowModal()
        infoDlg.Destroy() 
        
    def OnExit(self,e):
        self.Close(True)  # Close the frame.
        
app = wx.PySimpleApp()
frame=PyXPath(None,-1,'PyXPathToy')
app.MainLoop()


#file_uri = Uri.OsPathToUri('test.xml')
